@@ -1,5 +1,5 @@
 /* OpenVAS
-* $Id: openvassd.c 26738 2016-11-29 14:47:17Z kroosec $
+* $Id$
 * Description: Runs the OpenVAS-scanner.
 *
 * Authors: - Renaud Deraison <deraison@nessus.org> (Original pre-fork develoment)
@@ -59,6 +59,7 @@
 #include <openvas/base/kb.h>         /* for KB_PATH_DEFAULT */
 #include <openvas/base/gpgme_util.h>
 #include <openvas/misc/prefs.h>      /* for prefs_get() */
+#include <openvas/misc/vendorversion.h>      /* for prefs_get() */
 
 #include <gcrypt.h> /* for gcry_control */
 
@@ -71,6 +72,10 @@
 #include "utils.h"        /* for wait_for_children1 */
 #include "pluginlaunch.h" /* for init_loading_shm */
 
+#if GNUTLS_VERSION_NUMBER < 0x030300
+#include <openvas/misc/network.h>     /* openvas_SSL_init */
+#endif
+
 #ifdef SVN_REV_AVAILABLE
 #include "svnrevision.h"
 #endif
@@ -364,7 +369,6 @@ reload_openvassd ()
   prefs_config (config_file);
 
   /* Reload the plugins */
-  nvticache_free ();
   ret = plugins_init ();
   set_globals_from_preferences ();
   loading_handler_stop (handler_pid);
@@ -470,6 +474,138 @@ check_reload ()
     }
 }
 
+
+/**
+ * @brief Get the pid and ppid from /proc to find the running scan pids.
+ *        Send SIGUSR2 kill signal to all running scans to stop them.
+ */
+static void
+stop_all_scans (void)
+{
+  int i, ispid;
+  GDir *proc = NULL;
+  const gchar *piddir = NULL;
+  gchar *pidstatfn = NULL;
+  gchar **contents_split = NULL;
+  gchar *contents = NULL;
+  GError *error = NULL;
+  gchar *parentID = NULL;
+  gchar *processID = NULL;
+
+  proc = g_dir_open ("/proc", 0, &error);
+  if (error != NULL)
+  {
+    log_write ("Unable to open directory: %s\n", error->message);
+    g_error_free (error);
+    return;
+  }
+  while ((piddir = g_dir_read_name (proc)) != NULL)
+    {
+      ispid = 1;
+      for (i = 0; i < (int)strlen (piddir); i++)
+        if (!g_ascii_isdigit (piddir[i]))
+          {
+            ispid = 0;
+            break;
+          }
+      if (!ispid)
+        continue;
+
+      pidstatfn = g_strconcat ("/proc/", piddir, "/stat", NULL);
+      if (g_file_get_contents (pidstatfn, &contents, NULL, NULL))
+        {
+          contents_split = g_strsplit (contents," ", 6);
+          parentID = g_strdup (contents_split[3]);
+          processID = g_strdup (contents_split[0]);
+
+          g_free (pidstatfn);
+          pidstatfn = NULL;
+          g_free (contents);
+          contents  = NULL;
+          g_strfreev (contents_split);
+          contents_split  = NULL;
+
+          if (atoi(parentID) == (int)getpid())
+            {
+              log_write ("Stopping running scan with PID: %s", processID);
+              kill (atoi(processID), SIGUSR2);
+            }
+          g_free (parentID);
+          parentID = NULL;
+          g_free (processID);
+          processID = NULL;
+        }
+      else
+        {
+          g_free (pidstatfn);
+          pidstatfn = NULL;
+          continue;
+        }
+    }
+
+  if (proc)
+    g_dir_close (proc);
+}
+
+/**
+ * @brief Check if Redis Server is up and if the KB exists. If KB does not
+ * exist,force a reload and stop all the running scans.
+ */
+void
+check_kb_status ()
+{
+  int  waitredis = 5, waitkb = 5, ret = 0;
+  kb_t kb_access_aux;
+
+  while (waitredis != 0)
+    {
+      ret = kb_new (&kb_access_aux, prefs_get ("kb_location"));
+      if (ret)
+        {
+          log_write ("Redis connection lost. Trying to reconnect.");
+          waitredis--;
+          sleep (5);
+          continue;
+        }
+      else
+        {
+          kb_delete (kb_access_aux);
+          break;
+        }
+    }
+
+  if (waitredis == 0)
+    {
+      log_write ("Critical Redis connection error.");
+      exit (1);
+    }
+
+  while (waitkb != 0)
+    {
+      kb_access_aux = kb_find (prefs_get ("kb_location"), "nvticache");
+      if (!kb_access_aux)
+        {
+          log_write ("Redis kb not found. Trying again in 2 seconds.");
+          waitkb--;
+          sleep (2);
+          continue;
+        }
+      else
+        {
+          kb_lnk_reset (kb_access_aux);
+          g_free (kb_access_aux);
+          break;
+        }
+    }
+
+  if (waitredis != 5 || waitkb == 0)
+    {
+      log_write ("Redis connection error. Stopping all the running scans.");
+      stop_all_scans ();
+      reload_openvassd ();
+    }
+}
+
 static void
 main_loop ()
 {
@@ -483,20 +619,61 @@ main_loop ()
   proctitle_set ("openvassd: Waiting for incoming connections");
   for (;;)
     {
-      int soc;
+      int soc, opts;
       unsigned int lg_address;
       struct sockaddr_un address;
       struct arglist *globals;
+      fd_set set;
+      struct timeval timeout;
+      int rv;
 
       check_termination ();
       check_reload ();
+      check_kb_status ();
       wait_for_children1 ();
       lg_address = sizeof (struct sockaddr_un);
-      soc = accept (global_iana_socket, (struct sockaddr *) (&address),
+
+      /* Setting the socket to non-blocking and the use of select() for
+       * listen() before accept() is done only for openvas-9. It allows to
+       * go through the loop every 0.5sec without stuck in the accept() call.
+       * In the trunk version the manager ask the the scanner for new NVTs
+       * every 10sec, so the loop is not stuck in accept().
+       */
+      if ((opts = fcntl (global_iana_socket, F_GETFL, 0)) < 0)
+        {
+          log_write ("fcntl: %s", strerror (errno));
+          exit (0);
+        }
+      if (fcntl (global_iana_socket, F_SETFL, opts | O_NONBLOCK) < 0)
+        {
+          log_write ("fcntl: %s", strerror (errno));
+          exit (0);
+        }
+
+      if (listen (global_iana_socket, 5) < 0)
+        continue;
+
+      FD_ZERO(&set);
+      FD_SET(global_iana_socket, &set);
+
+      timeout.tv_sec = 0;
+      timeout.tv_usec = 500000;
+
+      rv = select(global_iana_socket + 1, &set, NULL, NULL, &timeout);
+      if(rv == -1) /* Select error. */
+        continue;
+      else if(rv == 0) /* Timeout. */
+        continue;
+      else
+        soc = accept (global_iana_socket, (struct sockaddr *) (&address),
                     &lg_address);
       if (soc == -1)
         continue;
 
+      /* Set the socket to blocking again. */
+      if (fcntl (global_iana_socket, F_SETFL, opts) < 0)
+        log_write ("fcntl: %s", strerror (errno));
+
       globals = g_malloc0 (sizeof (struct arglist));
       arg_add_value (globals, "global_socket", ARG_INT, GSIZE_TO_POINTER (soc));
 
@@ -657,9 +834,7 @@ flush_all_kbs ()
   if (rc)
     return rc;
 
-  rc = kb_flush (kb);
-
-  kb_delete (kb);
+  rc = kb_flush (kb, "nvticache");
   return rc;
 }
 
@@ -692,6 +867,7 @@ main (int argc, char *argv[])
   static gboolean display_version = FALSE;
   static gboolean dont_fork = FALSE;
   static gchar *config_file = NULL;
+  static gchar *vendor_version_string = NULL;
   static gchar *listen_owner = NULL;
   static gchar *listen_group = NULL;
   static gchar *listen_mode = NULL;
@@ -708,6 +884,8 @@ main (int argc, char *argv[])
      "Do not run in daemon mode but stay in foreground", NULL},
     {"config-file", 'c', 0, G_OPTION_ARG_FILENAME, &config_file,
      "Configuration file", "<filename>"},
+    {"vendor-version", '\0', 0, G_OPTION_ARG_STRING, &vendor_version_string,
+     "Use <string> as vendor version.", "<string>"},
     {"cfg-specs", 's', 0, G_OPTION_ARG_NONE, &print_specs,
      "Print configuration settings", NULL},
     {"sysconfdir", 'y', 0, G_OPTION_ARG_NONE, &print_sysconfdir,
@@ -774,6 +952,9 @@ main (int argc, char *argv[])
   if (gnupg_dir)
     set_gpghome (gnupg_dir);
 
+  if (vendor_version_string)
+    vendor_version_set (vendor_version_string);
+
   if (!config_file)
     config_file = OPENVASSD_CONF;
   if (only_cache)
@@ -802,10 +983,18 @@ main (int argc, char *argv[])
     }
   flush_all_kbs ();
 
+#if GNUTLS_VERSION_NUMBER < 0x030300
+  if (openvas_SSL_init () < 0)
+    log_write ("Could not initialize openvas SSL!");
+#endif
+
   // Daemon mode:
   if (dont_fork == FALSE)
     set_daemon_mode ();
   pidfile_create ("openvassd");
+
+  /* Ignore SIGHUP while reloading. */
+  openvas_signal (SIGHUP, SIG_IGN);
   handler_pid = loading_handler_start ();
   if (handler_pid < 0)
     return 1;
