@@ -1,5 +1,5 @@
 /* OpenVAS
-* $Id: pluginlaunch.c 26467 2016-10-24 12:18:36Z kroosec $
+* $Id$
 * Description: Manages the launching of plugins within processes.
 *
 * Authors: - Renaud Deraison <deraison@nessus.org> (Original pre-fork develoment)
@@ -197,7 +197,7 @@ update_running_processes (void)
                     }
                   if (log_whole)
                     {
-                      char *name = nvticache_get_name (oid);
+                      char *name = nvticache_get_filename (oid);
                       log_write
                         ("%s (%s) [%d] finished its job in %ld.%.3ld seconds",
                          name, oid, processes[i].pid,
